#!/usr/bin/env python

import csv
import itertools
import operator
import collections

FILE = 'AlgNoPort.csv'
#FILE = 'KirNoPort.csv'

RESULT = '%sRanking.%s' % FILE.partition('.')[::2]
SYMMETRIZE_CASELESS = True
INCLUDE_NONBLOCKING = False # True to show all cells
INCLUDE_THIRD_OVER_THIRD = False


def load(filename=FILE):
    with open(filename, 'Urb') as fd:
        csvfile = csv.reader(fd, delimiter='\t')
        header = [c.replace('.', '_') for c in next(csvfile)]
        header += 'blocks hierarchy'.split()
        Row = collections.namedtuple('Row', header)._make
        rows = [Row(c for c in row + ['', '']) for row in csvfile]
    return rows


def save(rows, filename=RESULT):
    with open(filename, 'wb') as fd:
        writer = csv.writer(fd)
        writer.writerow(rows[0]._fields)
        writer.writerows(rows)


def groupby(rows, attribute, mapping=False):
    key = operator.attrgetter(attribute)
    result = itertools.groupby(sorted(rows, key=key), key)
    if mapping:
        result = dict((k, list(g)) for k, g in result)
    return result


Contexts = collections.namedtuple('Contexts', 'A P')


def main():
    global result
    rows = load()
    result = []

    for lid, lid_rows in groupby(rows, 'LID'):
        print 'Language: %s' % lid
        for slot, slot_rows in groupby(lid_rows, 'Slot'):
            slot_rows = list(slot_rows)
            print 'Slot: %s' % slot
            markers = {}
            for fmtv, fmtv_rows in groupby(slot_rows, 'Fmtv_ID'):
                roles = groupby(fmtv_rows, 'Role', mapping=True)
                A, P = roles.get('Atr', []), roles.get('P', [])
                A, P = (sorted(set(r.PoS for r in AP)) for AP in (A, P))
                if SYMMETRIZE_CASELESS and A and P:
                    A = P = sorted(set(A + P))
                print '  Marker: %s\n    A: %s\n    P: %s' % (fmtv, ', '.join(A), ', '.join(P))
                markers[fmtv] = Contexts(A, P)
            for r in slot_rows:
                blocked = []
                hierarchies = []
                for fmtv, (A, P) in markers.iteritems():
                    if r.Fmtv_ID == fmtv:
                        pass
                    elif (r.co_Role == 'Atr' and r.co_PoS in A and r.PoS not in P
                        or
                        r.co_Role == 'P' and r.co_PoS in P and r.PoS not in A):
                        blocked.append(fmtv)
                        #print '%s\n\tblocks %s' % (r, fmtv)
                        hierarchies.append((int(r.PoS[0]), int(r.co_PoS[0])))
                if (INCLUDE_NONBLOCKING or blocked) and (INCLUDE_THIRD_OVER_THIRD
                    or set(hierarchies) != set([(3, 3)])):
                    result.append(r._replace(
                        blocks='; '.join(blocked),
                        hierarchy='; '.join('%s > %s' % h for h in set(hierarchies))))
                
        print

    save(result)

        
if __name__ == '__main__':
    main()
